@@ -4,7 +4,7 @@ walk.var = diag(c(0.005,0.005,0.05))
 cscale = 1
 
 prior = function(pars){ return( sum(dnorm(pars/c(1,1,10),log=TRUE))) }
-nstep = 20
+nstep = 50
 
 
-hmm = Colloc.MCMC(FhNtimes,FhNdata,FhNpars,coefs,lik,proc,prior,walk.var,cscale,nstep,in.meth='SplineEst',control.in=control.in)
+hmm = Colloc.MCMC(FhNtimes,FhNdata,FhNpars,Ires2$coefs,srklik,srkproc,prior,walk.var,cscale,nstep,in.meth='SplineEst',control.in=control.in)
